package edu.uoc.android.memoriesplanner;

import android.app.Fragment;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.PopupMenu;

import com.github.sundeepk.compactcalendarview.CompactCalendarView;
import com.github.sundeepk.compactcalendarview.domain.Event;
import com.orm.SugarContext;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.TimeZone;

import edu.uoc.android.memoriesplanner.model.EventContent;

/**
 * An activity representing the user events calendar. This activity
 * has different presentations for handset and tablet-size devices. On
 * handsets, the activity presents a calendar. When a day is touched
 * lead to a {@link EventDetailActivity} representing
 * item details, or to {@link NewEventActivity} with the option to create a new event.
 * On tablets, the activity presents the calendar and
 * item details side-by-side using two vertical panes.
 */

public class CalendarActivity extends AppCompatActivity {

    private CompactCalendarView calendarView;               //Calendar view
    private boolean mTwoPane;
    private List<EventContent.EventItem> eventList;         //List of events

    //Date format for the date on the toolbar
    private SimpleDateFormat dateFormatMonth = new SimpleDateFormat("MMMM-yyyy", Locale.getDefault());
    //Date format for the dates stored in the database
    private SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault());


    //ONCREATE METHOD
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_calendar);

        //Top toolbar
        final Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        toolbar.setTitle(dateFormatMonth.format(new Date()));
        setSupportActionBar(toolbar);

        if (findViewById(R.id.event_detail_container) != null) {
            // The detail container view will be present only in the
            // large-screen layouts (res/values-w900dp).
            // If this view is present, then the
            // activity should be in two-pane mode.
            mTwoPane = true;
        }

        //Get the calendar view and set the locale time zone
        calendarView = (CompactCalendarView) findViewById(R.id.compactcalendar_view);
        calendarView.setLocale(TimeZone.getDefault(), Locale.getDefault());

        //Get all the events from the database
        SugarContext.init(this);
        eventList = EventContent.getEvents();
        SugarContext.terminate();

        //Prepare the dates and event objects for the calendar
        Date dateStart = new Date();
        Date dateEnd = new Date();
        Long timeInMillisStart = 0L;
        Long timeInMillisEnd = 0L;
        Event calendarEventStart;
        Event calendarEventEnd;

        //Get all the events from the obtained list and add them to the calendar
        for (EventContent.EventItem event : eventList) {
            try {
                dateStart = dateFormat.parse(event.getInitialDate());
                dateEnd = dateFormat.parse(event.getFinalDate());
                timeInMillisStart = dateStart.getTime();
                timeInMillisEnd = dateEnd.getTime();

                calendarEventStart = new Event(R.color.colorPrimary, timeInMillisStart, event.getTitle());
                //If starting and ending day of the event are the same, only add the starting date
                if (!timeInMillisEnd.equals(timeInMillisStart)) {
                    calendarEventEnd = new Event(R.color.calendarEndDate, timeInMillisEnd, event.getTitle());
                    //Add the event end to the calendar
                    calendarView.addEvent(calendarEventEnd, false);
                }

                //Add the event start to the calendar
                calendarView.addEvent(calendarEventStart, false);
            } catch (Exception e) {
                System.out.println("Excep" + e);
            }
        }

        //Day click listener
        calendarView.setListener(new CompactCalendarView.CompactCalendarViewListener() {
            @Override
            public void onDayClick(Date dateClicked) {
                Context context = getApplicationContext();
                //Get all the events active on the clicked day
                List<Event> events = calendarView.getEvents(dateClicked);

                PopupMenu menu = new PopupMenu(calendarView.getContext(), calendarView);

                for (Event event : events){
                    final String eventTitle = event.getData().toString();

                    //Set the Onclick listener to the "Event name" Menu option.
                    //In twoPane mode, display the EventDetailFragment into the second panel of the screen
                    //In single panel mode, open the activity EventDetailActivity
                    menu.getMenu().add(eventTitle).setOnMenuItemClickListener(new MenuItem.OnMenuItemClickListener() {
                        @Override
                        public boolean onMenuItemClick(MenuItem menuItem) {

                            if (mTwoPane) {
                                Bundle arguments = new Bundle();
                                arguments.putString(EventDetailFragment.ARG_EVENT_TITLE, eventTitle);
                                Fragment fragment = new Fragment();
                                fragment.setArguments(arguments);
                                getFragmentManager().beginTransaction()
                                        .replace(R.id.event_detail_container, fragment)
                                        .commit();
                            } else {
                                Context context = calendarView.getContext();
                                Intent intent = new Intent(context, EventDetailActivity.class);
                                intent.putExtra(EventDetailFragment.ARG_EVENT_TITLE, eventTitle);

                                context.startActivity(intent);
                            }

                            return true;
                        }
                    });
                }

                //Set the Onclick listener to the "Add New Event" Menu option.
                //In twoPane mode, display the NewEventFragment into the second panel of the screen
                //In single panel mode, open the activity NewEventActivity
                menu.getMenu().add(R.string.title_activity_new_event).setOnMenuItemClickListener(new MenuItem.OnMenuItemClickListener(){
                    @Override
                    public boolean onMenuItemClick (MenuItem menuItem){
                        if (mTwoPane) {
                            NewEventFragment fragment = new NewEventFragment();
                            getSupportFragmentManager().beginTransaction()
                                    .replace(R.id.event_detail_container, fragment)
                                    .commit();
                        } else {
                            Context context = calendarView.getContext();
                            Intent intent = new Intent(context, NewEventActivity.class);
                            context.startActivity(intent);
                        }
                        return true;
                    }
                });
                //Display the menu
                menu.show();
            }

            //Change toolbar title when user moves through calendar months
            @Override
            public void onMonthScroll(Date firstDayOfNewMonth) {
                toolbar.setTitle(dateFormatMonth.format(firstDayOfNewMonth));
            }
        });
    }

    //ONCREATEOPTIONSMENU METHOD.
    //Set the top bar menu
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.main_menu, menu);
        return super.onCreateOptionsMenu(menu);
    }

    //ONOPTIONSITEMSELECTED METHOD
    //Navigation between activities using the toolbar menu
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        Intent navigation_intent;
        switch (item.getItemId()) {
            case R.id.future_events:
                navigation_intent = new Intent(this, EventFutureListActivity.class);
                startActivity(navigation_intent);
                this.finish();
                break;
            case R.id.past_events:
                navigation_intent = new Intent(this, EventPastListActivity.class);
                startActivity(navigation_intent);
                this.finish();
                break;
            case R.id.calendar:
                navigation_intent = new Intent(this, CalendarActivity.class);
                startActivity(navigation_intent);
                this.finish();
                break;
            default:
                break;
        }
        return true;
    }
}
