package edu.uoc.android.memoriesplanner;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.CardView;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;

import com.orm.SugarContext;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import edu.uoc.android.memoriesplanner.adapter.ListAdapter;
import edu.uoc.android.memoriesplanner.model.EventContent;

/**
 * An activity representing a list of Past Events. This activity
 * has different presentations for handset and tablet-size devices. On
 * handsets, the activity presents a list of items, which when touched,
 * lead to a {@link EventDetailActivity} representing
 * item details. On tablets, the activity presents the list of items and
 * item details side-by-side using two vertical panes.
 */

public class EventPastListActivity extends AppCompatActivity {

    /**
     * Whether or not the activity is in two-pane mode, i.e. running on a tablet
     * device.
     */
    private ListAdapter adapter;            //Adapter of the recycler view with the events
    private boolean mTwoPane;
    private CardView addEventCardView;      //Card containing an event of the list

    //ONCREATE METHOD
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_event_past_list);

        //Top toolbar
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        toolbar.setTitle(R.string.title_activity_event_past_list);
        setSupportActionBar(toolbar);

        if (findViewById(R.id.event_detail_container) != null) {
            // The detail container view will be present only in the
            // large-screen layouts (res/values-w900dp).
            // If this view is present, then the
            // activity should be in two-pane mode.
            mTwoPane = true;
        }

        //Prepare the recyclerview with the event list
        View recyclerView = findViewById(R.id.event_list);
        assert recyclerView != null;
        setupRecyclerView((RecyclerView) recyclerView);

        //Set the Onclick listener to the "Add New Event" View.
        //In twoPane mode, display the NewEventFragment into the second panel of the screen
        //In single panel mode, open the activity NewEventActivity
        addEventCardView = findViewById((R.id.add_event_view));
        addEventCardView.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View v) {

                if (mTwoPane) {
                    NewEventFragment fragment = new NewEventFragment();
                    getSupportFragmentManager().beginTransaction()
                            .replace(R.id.event_detail_container, fragment)
                            .commit();
                } else {
                    Context context = v.getContext();
                    Intent intent = new Intent(context, NewEventActivity.class);
                    context.startActivity(intent);
                }
            }
        });
    }

    //ONRESTART METHOD
    //Refresh the list with updated events when the activity restarts
    @Override
    protected void onRestart(){
        super.onRestart();
        View recyclerView = findViewById(R.id.event_list);
        assert recyclerView != null;
        setupRecyclerView((RecyclerView) recyclerView);
    }

    //ONCREATEOPTIONSMENU METHOD.
    //Set the top bar menu
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.main_menu, menu);
        return super.onCreateOptionsMenu(menu);
    }

    //ONOPTIONSITEMSELECTED METHOD
    //Navigation between activities using the toolbar menu
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        Intent navigation_intent;
        switch (item.getItemId()) {
            case R.id.future_events:
                navigation_intent = new Intent(this, EventFutureListActivity.class);
                startActivity(navigation_intent);
                this.finish();
                break;
            case R.id.past_events:
                navigation_intent = new Intent(this, EventPastListActivity.class);
                startActivity(navigation_intent);
                this.finish();
                break;
            case R.id.calendar:
                navigation_intent = new Intent(this, CalendarActivity.class);
                startActivity(navigation_intent);
                this.finish();
                break;
            default:
                break;
        }
        return true;
    }

    //SETUPRECYCLERVIEW METHOD
    //Setup of the recycler View
    private void setupRecyclerView(@NonNull RecyclerView recyclerView) {
        //Format of the dates stored in the data base
        SimpleDateFormat dateFormat= new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault());
        Date currentDate = Calendar.getInstance().getTime();
        Date eventDate = Calendar.getInstance().getTime();
        String eventDateString = "";

        try {
            //Set hours, minutes and seconds to zero
            currentDate = dateFormat.parse(dateFormat.format(currentDate));
        }catch (Exception e){
            System.out.println("Excep"+e);
        }

        //Load the whole list of events from the database
        SugarContext.init(this);
        List<EventContent.EventItem> pastEvents = EventContent.getEvents();
        SugarContext.terminate();

        //Iterate through events list and filter with the currentDate
        Iterator<EventContent.EventItem> iter = pastEvents.iterator();
        while (iter.hasNext()) {
            EventContent.EventItem item = iter.next();
            eventDateString = item.getFinalDate();

            try{
                eventDate = dateFormat.parse(eventDateString);
            }catch (Exception e){
                System.out.println("Excep"+e);
            }

            //Compare current Date with event Date and remove the event from the list if its a future or active event
            if (currentDate.before(eventDate) || currentDate.equals(eventDate)) iter.remove();
        }

        //Sets the adapter of the view to display the events list
        adapter = new ListAdapter(this, pastEvents, mTwoPane);
        recyclerView.setAdapter(adapter);
    }
}
