package edu.uoc.android.memoriesplanner;

import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.orm.SugarContext;
import com.squareup.picasso.Picasso;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;

import edu.uoc.android.memoriesplanner.model.EventContent;

import static android.app.Activity.RESULT_OK;


/**
 * A fragment representing the new event option screen.
 * This fragment is either contained in a {@link EventFutureListActivity}
 * in two-pane mode (on tablets) or a {@link NewEventActivity}
 * on handsets.
 */
public class NewEventFragment extends Fragment {

    private EventContent.EventItem mItem;           //The event content this fragment is presenting

    private EditText editTitle;                     //Event title input
    private EditText editPlace;                     //Event place input
    private EditText editDescription;               //Event description input
    private DatePicker datePickerInitial;           //Event initial date picker
    private DatePicker datePickerFinal;             //Event final date picker
    private ImageView photo;                        //Event main photo
    private ImageView photoDeleteButton;            //Event main photo delete button
    private TextView photoText;                     //"Add photo" hint text of the event main photo
    private ImageButton saveButton;                 //Save event changes button
    private ImageButton cancelButton;               //Cancel event changes button

    //Date format for the dates stored in the database
    private SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy", Locale.ENGLISH);

    private static int PICK_IMAGE_REQUEST = 1;      //Pick photo code
    private String imageUri = "";                   //Path of the event main photo

    /**
     * Mandatory empty constructor for the fragment manager to instantiate the
     * fragment (e.g. upon screen orientation changes).
     */
    public NewEventFragment() {
    }

    //ONCREATE METHOD
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    //ONCREATEVIEW METHOD
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.new_event, container, false);

        //Get the elements from the view
        editTitle = (EditText) rootView.findViewById(R.id.event_title);
        editPlace = (EditText) rootView.findViewById(R.id.event_place);
        editDescription = (EditText) rootView.findViewById(R.id.event_description);
        datePickerInitial = (DatePicker) rootView.findViewById(R.id.initial_date);
        datePickerFinal = (DatePicker) rootView.findViewById(R.id.final_date);
        photo = (ImageView) rootView.findViewById(R.id.photo);
        photoDeleteButton = (ImageView) rootView.findViewById(R.id.delete_button_image);
        photoText = (TextView) rootView.findViewById(R.id.add_photo_text);
        saveButton = (ImageButton) rootView.findViewById(R.id.save_button);
        cancelButton = (ImageButton) rootView.findViewById(R.id.cancel_button);

        //Set click listener on the event main photo.
        //When clicked, calls the device gallery and let the user pick a photo
        photo.setOnClickListener(new View.OnClickListener(){
           @Override
            public void onClick(View v){
               Intent galleryIntent = new Intent(Intent.ACTION_PICK, android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
               galleryIntent.setType("image/*");
               startActivityForResult(galleryIntent, PICK_IMAGE_REQUEST);
           }
        });

        //Set click listener on the event main photo delete button
        //Refresh the view of the main event photo
        photoDeleteButton.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View v){
                photo.setImageResource(R.mipmap.ic_add);
                photoDeleteButton.setVisibility(View.INVISIBLE);
                photoText.setVisibility(View.VISIBLE);
                imageUri = "";
            }
        });

        //Set click listener on the cancel button
        cancelButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                //In twoPane mode, restart the current activity.
                //In single pane mode, close the current activity and goes to the previous one
                if (getActivity() instanceof NewEventActivity) {
                    getActivity().finish();
                } else if (getActivity() instanceof EventFutureListActivity) {
                    Intent intent = new Intent(getActivity(), EventFutureListActivity.class);
                    startActivity(intent);
                    getActivity().finish();
                } else if (getActivity() instanceof EventPastListActivity) {
                    Intent intent = new Intent(getActivity(), EventPastListActivity.class);
                    startActivity(intent);
                    getActivity().finish();
                }else if(getActivity() instanceof  CalendarActivity){
                    Intent intent = new Intent(getActivity(), CalendarActivity.class);
                    startActivity(intent);
                    getActivity().finish();
                }
            }
        });

        //Set click listener on the save button
        saveButton.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View v) {
                Context context = v.getContext();
                //Create the dateStrings with the data obtained from datepickers
                String initialPickerDate = datePickerInitial.getDayOfMonth() + "/" + (1+datePickerInitial.getMonth()) + "/" + datePickerInitial.getYear();
                String finalPickerDate = datePickerFinal.getDayOfMonth() + "/" + (1+datePickerFinal.getMonth()) + "/" + datePickerFinal.getYear();
                boolean correctDates = false;

                //Parse the formed data strings and compare them
                try{
                    Calendar calendarInitial = Calendar.getInstance();
                    calendarInitial.setTime(dateFormat.parse(initialPickerDate));

                    Calendar calendarFinal = Calendar.getInstance();
                    calendarFinal.setTime(dateFormat.parse(finalPickerDate));

                    correctDates = calendarInitial.getTimeInMillis() <= calendarFinal.getTimeInMillis();
                }
                catch(Exception e) {
                    //java.text.ParseException: Unparseable date: Getting error
                    System.out.println("Excep"+e);
                }


                //Check if event title is empty
                if ((editTitle.getText().toString().trim().length() == 0)){
                    Toast toast = Toast.makeText(context, R.string.toast_title_empty, Toast.LENGTH_LONG);
                    toast.show();
                }else if (!correctDates){
                    //Check if the initial date is previous to the final date
                    Toast toast = Toast.makeText(context, R.string.toast_dates_error, Toast.LENGTH_LONG);
                    toast.show();
                }else{
                    //Create a new event item with the data obtained from the inputs
                    EventContent.EventItem newEvent = new EventContent.EventItem();
                    newEvent.setTitle(editTitle.getText().toString());
                    newEvent.setPlace(editPlace.getText().toString());
                    newEvent.setDescription(editDescription.getText().toString());
                    newEvent.setUrlImage(imageUri);
                    newEvent.setInitialDate(initialPickerDate);
                    newEvent.setFinalDate(finalPickerDate);
                    newEvent.setUrlImageList("");

                    //If the event doesn't exist in the database, insert it
                    SugarContext.init(context);
                    if (EventContent.exists(newEvent.getTitle())){
                        Toast toast = Toast.makeText(context, R.string.toast_text_event_found, Toast.LENGTH_LONG);
                        toast.show();
                    }else{
                        newEvent.save();
                        Toast toast = Toast.makeText(context, R.string.toast_text_save_event, Toast.LENGTH_LONG);
                        toast.show();
                    }
                    SugarContext.terminate();

                    //In twoPane mode, restart the current activity.
                    //In single pane mode, close the current activity and goes to the previous one
                    if (getActivity() instanceof NewEventActivity){
                        getActivity().finish();
                    }else if(getActivity() instanceof EventFutureListActivity){
                        Intent intent = new Intent(getActivity(), EventFutureListActivity.class);
                        startActivity(intent);
                        getActivity().finish();
                    }else if(getActivity() instanceof  EventPastListActivity){
                        Intent intent = new Intent(getActivity(), EventPastListActivity.class);
                        startActivity(intent);
                        getActivity().finish();
                    }else if(getActivity() instanceof  CalendarActivity){
                        Intent intent = new Intent(getActivity(), CalendarActivity.class);
                        startActivity(intent);
                        getActivity().finish();
                    }
                }
            }
        });

        //Hide keyboard when clicking outside the input elements
        rootView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                // Get the input method manager
                InputMethodManager inputMethodManager = (InputMethodManager)
                        view.getContext().getSystemService(Context.INPUT_METHOD_SERVICE);
                // Hide the soft keyboard
                inputMethodManager.hideSoftInputFromWindow(view.getWindowToken(),0);
            }
        });

        return rootView;
    }

    //ONACTIVITYRESULT METHOD
    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        try {
            //Pick image from gallery action
            if (requestCode == PICK_IMAGE_REQUEST && resultCode == RESULT_OK && null != data) {
                Uri selectedImage = data.getData();

                //Get the real path of the file
                String[] filePathColumn = { MediaStore.Images.Media.DATA };
                Cursor cursor = getActivity().getContentResolver().query(selectedImage, filePathColumn, null, null, null);
                cursor.moveToFirst();
                int columnIndex = cursor.getColumnIndex(filePathColumn[0]);
                imageUri = cursor.getString(columnIndex);
                cursor.close();

                //Set the imageview with the obtained path from the image gallery intent
                Picasso.with(getActivity()).load(selectedImage).centerCrop().fit().into(photo);
                photoText.setVisibility(View.INVISIBLE);
                photoDeleteButton.setVisibility(View.VISIBLE);
            }
        } catch (Exception e) {
            Toast toast = Toast.makeText(this.getActivity(), R.string.toast_image_error, Toast.LENGTH_LONG);
            toast.show();
        }
    }
}
